@@ -1,7 +1,7 @@
 /*
  *  csync2 - cluster synchronization tool, 2nd generation
  *  LINBIT Information Technologies GmbH <http://www.linbit.com>
- *  Copyright (C) 2004, 2005  Clifford Wolf <clifford@clifford.at>
+ *  Copyright (C) 2004, 2005, 2006  Clifford Wolf <clifford@clifford.at>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -70,6 +70,7 @@ int connect_to_host(const char *peername)
 	if ( conn_open(peername) ) return -1;
 
 	if ( use_ssl ) {
+#if HAVE_LIBGNUTLS_OPENSSL
 		conn_printf("SSL\n");
 		if ( read_conn_status(0, peername) ) {
 			csync_debug(1, "SSL command failed.\n");
@@ -78,6 +79,11 @@ int connect_to_host(const char *peername)
 		}
 		conn_activate_ssl(0);
 		conn_check_peer_cert(peername, 1);
+#else
+		csync_debug(0, "ERROR: Config request SSL but this csync2 is built without SSL support.\n");
+		csync_error_count++;
+		return -1;
+#endif
 	}
 
 	conn_printf("CONFIG %s\n", url_encode(cfgname));
@@ -278,7 +284,7 @@ void csync_update_file_mod(const char *peername,
 auto_resolve_entry_point:
 	csync_debug(1, "Updating %s on %s ...\n", filename, peername);
 
-	if ( lstat(prefixsubst(filename), &st) != 0 ) {
+	if ( lstat_strict(prefixsubst(filename), &st) != 0 ) {
 		csync_debug(0, "ERROR: Cant stat %s.\n", filename);
 		csync_error_count++;
 		goto got_error;
@@ -375,7 +381,7 @@ auto_resolve_entry_point:
 	if ( S_ISLNK(st.st_mode) ) {
 		char target[1024];
 		int rc;
-		rc = readlink(filename, target, 1023);
+		rc = readlink(prefixsubst(filename), target, 1023);
 		if ( rc >= 0 ) {
 			target[rc]=0;
 			conn_printf("MKLINK %s %s %s\n",
@@ -383,6 +389,9 @@ auto_resolve_entry_point:
 					url_encode(target));
 			if ( (last_conn_status = read_conn_status(filename, peername)) )
 				goto maybe_auto_resolve;
+		} else {
+			csync_debug(1, "File is a symlink but radlink() failed.\n", st.st_mode);
+			goto got_error;
 		}
 	} else
 	if ( S_ISSOCK(st.st_mode) ) {
@@ -390,6 +399,9 @@ auto_resolve_entry_point:
 				url_encode(key), url_encode(filename));
 		if ( (last_conn_status = read_conn_status(filename, peername)) )
 			goto maybe_auto_resolve;
+	} else {
+		csync_debug(1, "File type (mode=%o) is not supported.\n", st.st_mode);
+		goto got_error;
 	}
 
 	conn_printf("SETOWN %s %s %d %d\n",
@@ -476,7 +488,7 @@ maybe_auto_resolve:
 					if (remotedata == -1)
 						goto remote_file_has_been_removed;
 
-					if ( lstat(prefixsubst(filename), &sbuf) ) goto got_error_in_autoresolve;
+					if ( lstat_strict(prefixsubst(filename), &sbuf) ) goto got_error_in_autoresolve;
 
 					if (auto_method == CSYNC_AUTO_METHOD_YOUNGER ||
 					    auto_method == CSYNC_AUTO_METHOD_OLDER)
@@ -567,7 +579,7 @@ void csync_update_host(const char *peername,
 	 */
 	for (t = tl; t != 0; t = next_t) {
 		next_t = t->next;
-		if ( !lstat(prefixsubst(t->value), &st) != 0 && !csync_check_pure(t->value)) {
+		if ( !lstat_strict(prefixsubst(t->value), &st) != 0 && !csync_check_pure(t->value)) {
 			*last_tn = next_t;
 			t->next = tl_mod;
 			tl_mod = t;
@@ -649,7 +661,9 @@ int csync_diff(const char *myname, const char *peername, const char *filename)
 	while ( (g=csync_find_next(g, filename)) ) {
 		if ( !g->myname || strcmp(g->myname, myname) ) continue;
 		for (h = g->host; h; h = h->next)
-			if (!strcmp(h->hostname, peername)) goto found_host_check;
+			if (!strcmp(h->hostname, peername)) {
+				goto found_host_check;
+			}
 	}
 	csync_debug(0, "Host pair + file not found in configuration.\n");
 	csync_error_count++;
@@ -851,6 +865,17 @@ int csync_insynctest_all(int init_run, int auto_diff, const char *filename)
 	struct csync_group *g;
 	int ret = 1;
 
+	if (auto_diff && filename) {
+		struct peer *pl = csync_find_peers(filename, 0);
+		int pl_idx;
+
+		for (pl_idx=0; pl && pl[pl_idx].peername; pl_idx++)
+			csync_diff(pl[pl_idx].myname, pl[pl_idx].peername, filename);
+
+		free(pl);
+		return ret;
+	}
+
 	for (g = csync_group; g; g = g->next) {
 		if ( !g->myname ) continue;
 		for (myname=myname_list; myname; myname=myname->next)
